package producerconsumer;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

public class Monitor {

  // Number of items that can be stored in the buffer
  static final int BUFFER_SIZE = 1000;

  // Circular buffer shared by threads
  private Item buffer[] = new Item[BUFFER_SIZE];

  // Index of the next empty slot for an item
  private int emptySlot = 0;

  // Index of the next slot with an item to consume
  private int itemSlot  = 0;

  // Number of unconsumed items in the shared buffer
  private int count = 0;

  public synchronized void insertItem(Item item)
  // Precondition:  call to this function must be thread safe, it
  //                accesses shared data!!!
  // Postcondition: on return, an item has been placed in the item buffer
  //                and the position of the next empty slot has been updated
  {
    // If the buffer is full, then sleep, wakeup when a slot is available
    if (count == BUFFER_SIZE) sleep();

    buffer[emptySlot] = item;
    emptySlot = (emptySlot + 1) % BUFFER_SIZE;
    count++;

    // If the buffer was empty, use notify to wakeup sleeping removeItem()
    if (count==1) notify();
  }

  public synchronized Item removeItem()
  // Precondition:  call to this function must be thread safe, it
  //                accesses shared data!!!
  // Postcondition: on return an item has been removed from the item buffer
  //                and the position of the next item slot has been updated
  {
    Item result;

    // If the buffer is empty, then sleep, wakeup when we have an item
    if (count == 0) sleep();

    result = buffer[itemSlot];
    itemSlot = (itemSlot + 1) % BUFFER_SIZE;
    count--;

    // If the buffer was FULL, use notify to wakeup sleeping insertItem()
    if (count == (BUFFER_SIZE-1)) notify();

    return result;
  }

  private void sleep()
  {
    try {
      wait();
    }
    catch (Exception e)
    {
    }
  }

}